<?php
/**
 * Persistent ID Component
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
class PersistentIdComponent extends Object {
	/**
	 * Creates a persistent ID
	 * It is a unique value.
	 * @param $accountid account ID
	 * @param $entityId SP's entity ID
	 * @return Persistent ID
	 */
	function create($accountid, $entityId) {
		$salt = Configure::read("salt");
		$sha1 = sha1($entityId . "!" . $accountid . "!" . $salt, true);
		$base64 = base64_encode($sha1);
		return $base64;
	}

	/**
	 * generate random persistend ID
	 * @param $entityId SP's entity ID
	 * @return Persistent ID
	 */
	function randomUUID($entityId) {
		$salt = Configure::read("salt");
		$accountid = sha1(rand());
		$sha1 = sha1($entityId . "!" . $accountid . "!" . $salt, true);
		$base64 = base64_encode($sha1);
		return $base64;
	}

	/**
	 * Create eduPersonTargetedID
	 * @param string $spEntityId SP's entity ID
	 * @param string $persistentId Persistent ID
	 * @return eduPersonTargetedID
	 */
	function createEduPersonTargetedID($spEntityId, $persistentId) {
		$mAPEntityID = Configure::read("entityID");
		$ePTID = $mAPEntityID.'!'.$spEntityId.'!'.$persistentId;
		return $ePTID;
	}
}
?>