<?php
/**
 * People API Controller
 *
 * PHP versions 5
 *
 * @copyright     Copyright (c) 2011 - 2012, National Institute of Informatics,
 * @license       BSD License (http://creativecommons.org/licenses/BSD/)
 */
App::import('Controller', 'Api');

class PeopleController extends ApiController {
	var $name = 'People';
	var $components = array('RequestHandler', 'PersistentId');
	var $uses = array('Account', 'Mygroup', 'Group', 'GroupSphost', 'AddGroup', 'SpHost', 'Shibpid', 'Agreement','AgreementIsmemberof', 'AccountEppn');

	function beforeRender(){
		Configure::write('debug',0);
		$this->RequestHandler->setContent('json');
		$this->RequestHandler->respondAs('application/json; charset=UTF-8');
		parent::beforeRender();
	}

	/**
	 * excute people API
	 * @param $find @all:全員 @member:メンバーのみ
	 * @param $groupKey 検索対象のグループKEY
	 * 使用例)  https://xxxx/api/people/@all/グループKEY
	 */
	function index($find = null) {
		$lang = array_key_exists('lang',$this->params['url'])? $this->params['url']['lang'] : 'en';
		$this->apiInfoLog('people  find='.$find. '  url='.Router::url(null,true) . ' lang='.$lang);

		if ($find == null || ($find != '@me')) {
			$this->apiErrorLog('not find parameter(find)');
			$this->cakeError('jsonBadRequest');
			return;
		}

		// groupKey を URL で分割する
		// %2f 文字を含む Parameter は CakePHP では取得できない為、URL から直接取り出す
		$groupKey = str_replace($find,'',strstr(Router::url(null,true),$find));
		$this->apiInfoLog('groupKey url is ' . $groupKey);
		if (($groupKey === '') || ($groupKey === '/')){
			$this->apiErrorLog('not find group id(key)');
			$this->cakeError('jsonNotFound');
			return;
		}

		// URL の最後が admin であれば admin
		$groupUrl = $groupKey;
		$groupKeyParts = explode('/',$groupKey);
		$groupKey = array_pop($groupKeyParts);
		$isAdmin = true;
		if ($groupKey === 'admin') $groupKey = array_pop($groupKeyParts);
		else $isAdmin = false;
		$this->apiInfoLog('groupKey is ' . $groupKey);

		$group = array();
		if (3 <= count($groupKeyParts)){
			$selfUrl = Configure::read("entityID");
			$parts = explode('/',$groupUrl);
			$checkUrl = $parts[1] . "//" . $parts[2];
			if (false !== strpos($selfUrl , $checkUrl)){
				if ("gr" === $parts[3]){
					$group = $this->Group->find('first',
						array('fields' => array('id', 'group_key', 'introduction','sp'),
						'conditions' => array('group_key' => $groupKey , 'sp' => 0)));
				//} else if ("sp" === $parts[3]){
				//	$group = $this->Group->find('first',
				//		array('fields' => array('id', 'group_key', 'introduction','sp'),
				//		'conditions' => array('group_key' => $groupKey , 'sp' => 1)));
				} else {
					$this->apiErrorLog('not find group id(key)');
					$this->cakeError('jsonNotFound');
					return;
				}
			}
		} else {
			$group = $this->Group->find('first',
					array('fields' => array('id', 'group_key', 'introduction','sp'),
					'conditions' => array('group_key' => $groupKey)));
		}
		if (empty($group)) {
			$this->apiErrorLog('Group does not exist.');
			$this->cakeError('jsonNotFound');
		}

		// 上位グループ(このグループと接続しているグループ、SPコネクタ)を取得
		$upGroupIdList = $this->AddGroup->find('list', array('fields' => array('top_id'),
			'conditions' => array('bottom_id' => $group['Group']['id'])));
		// 上位グループと接続しているSPコネクタを取得
$this->apiInfoLog('group is ' . var_export($group,true));
$this->apiInfoLog("upGroupIdList is " . var_export($upGroupIdList,true));
		$upSPCIdList = $this->AddGroup->find('list', array('fields' => array('top_id'),
			'conditions' => array('bottom_id' => $upGroupIdList)));
$this->apiInfoLog("upSPCIdList is " . var_export($upSPCIdList,true));
		// SPが利用しているSPコネクタを取得
		// SPを取得
		$spHost = $this->getSpHost();
$this->apiInfoLog("spHost is " . var_export($spHost,true));
		// SPが利用しているSPコネクタを取得
		$spcIdList = $this->GroupSphost->find('list', array('fields' => array('group_id'),
			'conditions' => array('sp_id' => $spHost['SpHost']['id'])));
$this->apiInfoLog("spcIdList is " . var_export($spcIdList,true));
		// 該当グループがSPコネクタと接続しているかを確認
		$validateConnection = false;
		if (!$this->validateConnect($spcIdList, $upGroupIdList)) {

			if (($group['Group']['sp']) && (in_array($group['Group']['id'],$spcIdList,true))){
$this->apiInfoLog("this group is sp connector");
			} else {
				$this->apiErrorLog('not connect sp connector');
				$this->cakeError('jsonCannotAccess');
				return;
			}
		}

		// SP && 管理者の指定以外はこのグループの下位グループを取得
		$bottomGroupIdList = array();
		if (!(($group['Group']['sp']) && ($isAdmin)))
			$bottomGroupIdList = $this->getBottomGroups($group['Group']['id']);

		// 検索対象となるグループID一覧を生成
		$groupIdList = array();
$this->apiInfoLog("groupIdList before is " . var_export($groupIdList,true));
		array_push($groupIdList, $group['Group']['id']);
		foreach($bottomGroupIdList as $id) {
			array_push($groupIdList, $id);
		}
$this->apiInfoLog("groupIdList after is " . var_export($groupIdList,true));

		$condition = array();
		$this->apiInfoLog('isAdmin is ' . $isAdmin);
		if ($isAdmin) {
			$condition = array('groupid' => $groupIdList, 'admin' => array(1, 2));
		} else {
			$condition = array('groupid' => $groupIdList, 'admin' => array(0, 2));
		}

		$allAccountIdList = $this->Mygroup->find('all', array('fields' => array('groupid','account_id'),
			'conditions' => $condition));
		$accountIdList = array();
		foreach($allAccountIdList as $i)
			if ($this->isPermittedIsMemberOf($i , $spcIdList))
				array_push($accountIdList , $i['Mygroup']['account_id']);

		$shibpidList = $this->Shibpid->find('all', array(
			'fields' => array('localId', 'persistentId'),
			'conditions' => array(
				'localId' => $accountIdList,
				'peerEntity' => $spHost['SpHost']['entityid'],
				'deactivationDate' => NULL)));
		$peoples = $this->Account->find('all', array(
			'fields' => array('id', 'name', 'name_en',  'mail', 'language',
				'university', 'university_en', 'introduction', 'introduction_en'),
			'conditions' => array('id' => $accountIdList)));
		$accountEppns = $this->AccountEppn->find('all', array(
                        'fields' => array('account_id', 'eppn'),
                        'conditions' => array('account_id' => $accountIdList)));

		for ($i = 0; $i < count($peoples); $i++) {
			$peoples[$i]['Account']['entityId'] = array();
			$existEPTID = false;
			foreach ($shibpidList as $shibpid) {
				if ($peoples[$i]['Account']['id'] == $shibpid['Shibpid']['localId']) {
					$ePTID = $this->PersistentId->createEduPersonTargetedID($spHost['SpHost']['entityid'], $shibpid['Shibpid']['persistentId']);
					if (!$existEPTID){
						$peoples[$i]['Account']['eduPersonTargetedID'] = $ePTID;
						$existEPTID = true;
					}
					$peoples[$i]['Account']['entityId'][] = $spHost['SpHost']['entityid'];
				}
			}

			if (!$existEPTID) {
				$persistendId = $this->PersistentId->create($peoples[$i]['Account']['id'], $spHost['SpHost']['entityid']);
				// infinite loop blocking
				for ($j = 0; $j < 20; $j++) {
					if (!$existEPTID) {
						$newShibpid = array();
						$newShibpid['Shibpid']['localEntity'] = Configure::read("entityID");;
						$newShibpid['Shibpid']['peerEntity'] = $spHost['SpHost']['entityid'];
						$newShibpid['Shibpid']['principalName'] = $peoples[$i]['Account']['id'];
						$newShibpid['Shibpid']['localId'] = $peoples[$i]['Account']['id'];
						$newShibpid['Shibpid']['persistentId'] = $persistendId;
						$newShibpid['Shibpid']['creationDate'] = date('Y-m-d H:i:s');
						$this->Shibpid->create();
						$result = $this->Shibpid->save($newShibpid);
						if (empty($result)) {
							$errorKey = Configure::read("db_duplicate_error_key");
							// ckeck duplicate Error
							if (preg_match($errorKey, $this->Shibpid->lastError())) {
								$persistendId = $this->PersistentId->randomUUID($spHost['SpHost']['entityid']);
								continue;
							} else {
								// to following account
								$error = true;
								break;
							}
						}
						$ePTID = $this->PersistentId->createEduPersonTargetedID($spHost['SpHost']['entityid'], $persistendId);
						$peoples[$i]['Account']['eduPersonTargetedID'] = $ePTID;
						$existEPTID = true;
					}
					$peoples[$i]['Account']['entityId'][] = $spHost['SpHost']['entityid'];
				}
			}
			$peoples[$i]['Account']['eppn'] = array();
			foreach ($accountEppns as $accountEppn) {
				if ($peoples[$i]['Account']['id'] == $accountEppn['AccountEppn']['account_id']) {
					$peoples[$i]['Account']['eppn'][] = $accountEppn['AccountEppn']['eppn'];
				}
			}

		}
		// アカウントが同意している属性のみ設定する
		// SPが利用しているSPコネクタの同意情報を取得
$this->apiInfoLog('spcIdList is ' . var_export($spcIdList,true));
		$agreeList = $this->Agreement->find('all', array(
			'conditions' => array('group_id' => $spcIdList),
			'order' => array('account_id')));
//$this->apiInfoLog("before removeAttribute peoples is " . var_export($peoples , true));
		$peoples = $this->removeAttribute($peoples, $agreeList);
//$this->apiInfoLog("spHost         is " . var_export($sphost['SpHost']['entityid'],true));

//$this->apiInfoLog("peoples is " . var_export($peoples , true));
		$this->set("totalResults", count($peoples));
		$this->set("groupID", $group['Group']['id']);
		$this->set("groupKey", $group['Group']['group_key']);
		$this->set("groupExplanation", $group['Group']['introduction']);
		$this->set("peoples", $peoples);
		$this->set("lang",$lang);
	}

	/**
	 * isMemberOf 送出が同意されているかどうかを取得する
	 */
	private function isPermittedIsMemberOf($account , $spcIdList){
		$permitList = $this->AgreementIsmemberof->find('list',array(
			'fields' => array('ismemberof'),
			'conditions' => array('group_id' => $account['Mygroup']['groupid'],
				'account_id' => $account['Mygroup']['account_id'],
				'spc_id' => $spcIdList)));
		if (empty($permitList)) return false;
		return array_shift($permitList);
	}

	/**
	 * 下位グループを再帰的に取得する
	 */
	private function getBottomGroups($groupId){
		$bottomGroupIdList = $this->AddGroup->find('list', array('fields' => array('bottom_id'),
			'conditions' => array('top_id' => $groupId)));
		$groupIdList = $bottomGroupIdList;

		foreach($bottomGroupIdList as $i)
			$groupIdList = array_merge($groupIdList,$this->getBottomGroups($i));

		return $groupIdList;
	}

	private function validateConnect($spcIdList, $idList) {
		foreach ($spcIdList as $spcId) {
			foreach($idList as $id) {
				if ($spcId == $id) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * 同意していない属性を削除する。
	 * @param unknown_type $peoples
	 * @param unknown_type $agreeList
	 */
	private function removeAttribute(&$peoples, $agreeList) {
		$tmp_peoples = array();
		$count = count($peoples);
//$this->apiInfoLog("agreeList is " . var_export($agreeList,true));
		for ($i = 0; $i < $count; $i++) {

			$peopleAgree = array();
			foreach ($agreeList as $agree) {
				// 複数 SP にまたがる場合の同意は論理和
				if ($peoples[$i]['Account']['id'] == $agree['Agreement']['account_id']) {
					foreach(array_keys($agree['Agreement']) as $key){
						if ('created' === $key) continue;
						if ('modified' === $key) continue;
						if (!array_key_exists($key,$peopleAgree))
							 $peopleAgree[$key] = $agree['Agreement'][$key];
						else $peopleAgree[$key] |= $agree['Agreement'][$key];
					}
				}
			}
			//$this->apiInfoLog('peopleAgree of ' . $peoples[$i]['Account']['id'] . ' - ' . $peoples[$i]['Account']['name'] . ' is ' . var_export($peopleAgree,true));

			$removeCount = 0;
			if ((array_key_exists('eptid',$peopleAgree)) && (!$peopleAgree['eptid'])) {
				$peoples[$i]['Account']['eduPersonTargetedID'] = '';
				$removeCount++;
			}
			if ((array_key_exists('name',$peopleAgree)) && (!$peopleAgree['name'])) {
				$peoples[$i]['Account']['name'] = '';
				$peoples[$i]['Account']['name_en'] = '';
				$removeCount++;
			}
			if ((array_key_exists('mail',$peopleAgree)) && (!$peopleAgree['mail'])) {
				$peoples[$i]['Account']['mail'] = '';
				$removeCount++;
			}
			if ((array_key_exists('eppn',$peopleAgree)) && (!$peopleAgree['eppn'])) {
				$peoples[$i]['Account']['eppn'] = '';
				$removeCount++;
			}
			if ((array_key_exists('introduction',$peopleAgree)) && (!$peopleAgree['introduction'])) {
				$peoples[$i]['Account']['introduction'] = '';
				$peoples[$i]['Account']['introduction_ne'] = '';
				$removeCount++;
			}
			if ((array_key_exists('language',$peopleAgree)) && (!$peopleAgree['language'])) {
				$peoples[$i]['Account']['language'] = '';
				$removeCount++;
			}
			if ((array_key_exists('idp',$peopleAgree)) && (!$peopleAgree['idp'])) {
				$peoples[$i]['Account']['entityId'] = '';
				$peoples[$i]['Account']['university'] = '';
				$peoples[$i]['Account']['university_en'] = '';
				$removeCount++;
			}

			if ((empty($peopleAgree)) || ($removeCount >= 7)) {
				$this->apiInfoLog("user " . $peoples[$i]['Account']['id'] . " is skip");
				continue;
			}
			array_push($tmp_peoples,$peoples[$i]);
		}
		return $tmp_peoples;
	}
}
?>
